@@ -2,17 +2,12 @@ from __future__ import annotations
 
 import os
 import shlex
-import shutil
 import sys
 import tempfile
 from abc import abstractmethod
 from contextlib import contextmanager
 from pathlib import Path
-from typing import IO, TYPE_CHECKING, Any
-
-from rich.progress import Progress
-
-from ._util import SingletonContextManager
+from typing import TYPE_CHECKING, Any
 
 if sys.version_info >= (3, 9):
     from collections.abc import (
@@ -33,25 +28,31 @@ else:
         Sequence,
     )
 
+if TYPE_CHECKING:
+    from io import TextIOBase
+
+from rich.progress import Progress
+
+from ._util import SingletonContextManager
+
 if sys.version_info >= (3, 11):
     from typing import Self
 else:
     from typing_extensions import Self
 
-from ._base import FoamCaseBase
-from ._subprocess import DEVNULL, STDOUT
+if sys.version_info >= (3, 12):
+    from typing import override
+else:
+    from typing_extensions import override
+
+from ._subprocess import DEVNULL, STDOUT, LogFileMonitor
+from .base import FoamCaseBase
 
 if TYPE_CHECKING:
     from .._files import FoamFieldFile
 
 
 class FoamCaseRunBase(FoamCaseBase):
-    """
-    Abstract base class of :class:`FoamCase` and :class:`AsyncFoamCase`.
-
-    Do not use this class directly: use :class:`FoamCase` or :class:`AsyncFoamCase` instead.
-    """
-
     class TimeDirectory(FoamCaseBase.TimeDirectory):
         @abstractmethod
         def cell_centers(
@@ -59,6 +60,7 @@ class FoamCaseRunBase(FoamCaseBase):
         ) -> FoamFieldFile | Coroutine[None, None, FoamFieldFile]:
             raise NotImplementedError
 
+        @override
         @property
         @abstractmethod
         def _case(self) -> FoamCaseRunBase:
@@ -80,16 +82,17 @@ class FoamCaseRunBase(FoamCaseBase):
 
     __progress = SingletonContextManager(Progress)
 
-    def __delitem__(self, key: int | float | str) -> None:
-        shutil.rmtree(self[key].path)
-
     @staticmethod
     @abstractmethod
     def _run(
         cmd: Sequence[str | os.PathLike[str]] | str,
         *,
         cpus: int,
-        **kwargs: Any,
+        case: os.PathLike[str],
+        check: bool = True,
+        stdout: int | TextIOBase = DEVNULL,
+        stderr: int | TextIOBase = STDOUT,
+        process_stdout: Callable[[str], None] = lambda _: None,
     ) -> None | Coroutine[None, None, None]:
         raise NotImplementedError
 
@@ -277,7 +280,7 @@ class FoamCaseRunBase(FoamCaseBase):
     @contextmanager
     def __output(
         self, cmd: Sequence[str | os.PathLike[str]] | str, *, log: bool
-    ) -> Generator[tuple[int | IO[str], int | IO[str]], None, None]:
+    ) -> Generator[tuple[int | TextIOBase, int | TextIOBase], None, None]:
         if log:
             with (self.path / f"log.{self.__cmd_name(cmd)}").open("a") as stdout:
                 yield stdout, STDOUT
@@ -317,7 +320,13 @@ class FoamCaseRunBase(FoamCaseBase):
                 else:
                     progress.update(task)
 
-            yield process_stdout
+            # Set up log file monitoring
+            with LogFileMonitor(self.path, process_stdout) as log_monitor:
+                yield process_stdout
+
+                # Check for any final progress updates from log files
+                log_monitor.monitor_once()
+
             progress.update(task, completed=1, total=1)
 
     def __mkrundir(self) -> Path:
@@ -329,7 +338,7 @@ class FoamCaseRunBase(FoamCaseBase):
 
     def _copy_calls(
         self, dst: os.PathLike[str] | str | None
-    ) -> Generator[Any, None, Self]:
+    ) -> Generator[object | Coroutine[None, None, object], None, Self]:
         if dst is None:
             dst = self.__mkrundir()
 
@@ -337,10 +346,10 @@ class FoamCaseRunBase(FoamCaseBase):
 
         return type(self)(dst)
 
-    def _clean_calls(self, *, check: bool) -> Generator[Any, None, None]:
-        script_path = self.__clean_script()
-
-        if script_path is not None:
+    def _clean_calls(
+        self, *, check: bool
+    ) -> Generator[object | Coroutine[None, None, object], None, None]:
+        if (script_path := self.__clean_script()) is not None:
             yield self.run([script_path], cpus=0, check=check, log=False)
         else:
             for p in self.__clean_paths():
@@ -365,29 +374,31 @@ class FoamCaseRunBase(FoamCaseBase):
 
         return type(self)(dst)
 
-    def _restore_0_dir_calls(self) -> Generator[Any, None, None]:
+    def _restore_0_dir_calls(
+        self,
+    ) -> Generator[object | Coroutine[None, None, object], None, None]:
         yield self._rmtree(self.path / "0", ignore_errors=True)
         yield self._copytree(self.path / "0.orig", self.path / "0", symlinks=True)
 
     def _block_mesh_calls(
         self, *, check: bool, log: bool
-    ) -> Generator[Any, None, None]:
+    ) -> Generator[object | Coroutine[None, None, object], None, None]:
         yield self.run(["blockMesh"], cpus=0, check=check, log=log)
 
     def _decompose_par_calls(
         self, *, check: bool, log: bool
-    ) -> Generator[Any, None, None]:
+    ) -> Generator[object | Coroutine[None, None, object], None, None]:
         yield self.run(["decomposePar"], cpus=0, check=check, log=log)
 
     def _reconstruct_par_calls(
         self, *, check: bool, log: bool
-    ) -> Generator[Any, None, None]:
+    ) -> Generator[object | Coroutine[None, None, object], None, None]:
         yield self.run(["reconstructPar"], cpus=0, check=check, log=log)
 
-    def _prepare_calls(self, *, check: bool, log: bool) -> Generator[Any, None, None]:
-        script_path = self.__prepare_script()
-
-        if script_path is not None:
+    def _prepare_calls(
+        self, *, check: bool, log: bool
+    ) -> Generator[object | Coroutine[None, None, object], None, None]:
+        if (script_path := self.__prepare_script()) is not None:
             yield self.run([script_path], log=log, check=check)
 
         elif (self.path / "system" / "blockMeshDict").is_file():
@@ -397,12 +408,12 @@ class FoamCaseRunBase(FoamCaseBase):
         self,
         cmd: Sequence[str | os.PathLike[str]] | str | None = None,
         *,
+        cpus: int | None = None,
         parallel: bool | None,
-        cpus: int | None,
         check: bool,
         log: bool,
         **kwargs: Any,
-    ) -> Generator[Any, None, None]:
+    ) -> Generator[object | Coroutine[None, None, object], None, None]:
         if cmd is not None:
             if parallel:
                 if cpus is None:
@@ -438,54 +449,51 @@ class FoamCaseRunBase(FoamCaseBase):
                     **kwargs,
                 )
 
+        elif (script_path := self.__run_script(parallel=parallel)) is not None:
+            if parallel or parallel is None:
+                if cpus is None:
+                    if self._nprocessors > 0:
+                        cpus = self._nprocessors
+                    elif (self.path / "system" / "decomposeParDict").is_file():
+                        cpus = self._nsubdomains
+                    else:
+                        cpus = 1
+            elif cpus is None:
+                cpus = 1
+
+            yield self.run(
+                [script_path], parallel=False, cpus=cpus, check=check, **kwargs
+            )
+
         else:
-            script_path = self.__run_script(parallel=parallel)
+            yield self._prepare(check=check, log=log)
 
-            if script_path is not None:
-                if parallel or parallel is None:
-                    if cpus is None:
-                        if self._nprocessors > 0:
-                            cpus = self._nprocessors
-                        elif (self.path / "system" / "decomposeParDict").is_file():
-                            cpus = self._nsubdomains
-                        else:
-                            cpus = 1
-                elif cpus is None:
-                    cpus = 1
+            if not self and (self.path / "0.orig").is_dir():
+                yield self.restore_0_dir()
 
-                yield self.run(
-                    [script_path], parallel=False, cpus=cpus, check=check, **kwargs
+            if parallel is None:
+                parallel = (
+                    (cpus is not None and cpus > 1)
+                    or self._nprocessors > 0
+                    or (self.path / "system" / "decomposeParDict").is_file()
                 )
 
-            else:
-                yield self._prepare(check=check, log=log)
+            if parallel:
+                if (
+                    self._nprocessors == 0
+                    and (self.path / "system" / "decomposeParDict").is_file()
+                ):
+                    yield self.decompose_par(check=check)
 
-                if not self and (self.path / "0.orig").is_dir():
-                    yield self.restore_0_dir()
+                if cpus is None:
+                    cpus = max(self._nprocessors, 1)
+            elif cpus is None:
+                cpus = 1
 
-                if parallel is None:
-                    parallel = (
-                        (cpus is not None and cpus > 1)
-                        or self._nprocessors > 0
-                        or (self.path / "system" / "decomposeParDict").is_file()
-                    )
-
-                if parallel:
-                    if (
-                        self._nprocessors == 0
-                        and (self.path / "system" / "decomposeParDict").is_file()
-                    ):
-                        yield self.decompose_par(check=check)
-
-                    if cpus is None:
-                        cpus = max(self._nprocessors, 1)
-                elif cpus is None:
-                    cpus = 1
-
-                yield self.run(
-                    [self.application],
-                    parallel=parallel,
-                    cpus=cpus,
-                    check=check,
-                    **kwargs,
-                )
+            yield self.run(
+                [self.application],
+                parallel=parallel,
+                cpus=cpus,
+                check=check,
+                **kwargs,
+            )
