@@ -1,7 +1,8 @@
 from __future__ import annotations
 
+import contextlib
 import sys
-from typing import overload
+from typing import TYPE_CHECKING, overload
 
 if sys.version_info >= (3, 9):
     from collections.abc import Mapping
@@ -9,47 +10,70 @@ else:
     from typing import Mapping
 
 import numpy as np
+from multicollections import MultiDict
 
-from ._parsing import loads
-from ._types import (
-    Data,
-    DataLike,
-    Dimensioned,
-    DimensionSet,
-    KeywordEntryLike,
-    StandaloneData,
-    StandaloneDataLike,
-    SubDict,
-    SubDictLike,
-    is_sequence,
-)
+from ._parsing import Parsed
+from ._util import as_dict_check_unique, is_sequence
+from .types import Dimensioned, DimensionSet
+
+if TYPE_CHECKING:
+    from ._typing import (
+        Data,
+        DataLike,
+        KeywordEntryLike,
+        StandaloneData,
+        StandaloneDataLike,
+        SubDict,
+        SubDictLike,
+    )
 
 
 @overload
-def normalize_data(
-    data: DataLike, *, keywords: tuple[str, ...] | None = None
+def normalize(
+    data: DataLike, *, keywords: tuple[str, ...] | None = None, bool_ok: bool = True
 ) -> Data: ...
 
 
 @overload
-def normalize_data(
-    data: StandaloneDataLike, *, keywords: tuple[str, ...] | None = None
+def normalize(
+    data: StandaloneDataLike,
+    *,
+    keywords: tuple[str, ...] | None = None,
+    bool_ok: bool = True,
 ) -> StandaloneData: ...
 
 
 @overload
-def normalize_data(
-    data: SubDictLike, *, keywords: tuple[str, ...] | None = None
+def normalize(
+    data: SubDictLike, *, keywords: tuple[str, ...] | None = None, bool_ok: bool = True
 ) -> SubDict: ...
 
 
-def normalize_data(
+def normalize(
     data: DataLike | StandaloneDataLike | SubDictLike,
     *,
     keywords: tuple[str, ...] | None = None,
+    bool_ok: bool = True,
 ) -> Data | StandaloneData | SubDict:
     if isinstance(data, Mapping):
-        return {normalize_keyword(k): normalize_data(v) for k, v in data.items()}  # type: ignore [arg-type, misc]
+        items = ((normalize(k, bool_ok=False), normalize(v)) for k, v in data.items())
+        if keywords is None:
+            return as_dict_check_unique(items)
+        ret1: SubDict = MultiDict(items)
+        seen = set()
+        for k, v in ret1.items():
+            if k.startswith("#"):
+                if isinstance(v, Mapping):
+                    msg = f"Directive {k} cannot have a dictionary as value"
+                    raise ValueError(msg)
+            else:
+                if k in seen:
+                    msg = (
+                        f"Duplicate keyword {k} in dictionary with keywords {keywords}"
+                    )
+                    raise ValueError(msg)
+                seen.add(k)
+        return ret1
 
     if keywords == () and is_sequence(data) and not isinstance(data, tuple):
         try:
@@ -88,17 +112,17 @@ def normalize_data(
                 if arr.ndim == 1 or (arr.ndim == 2 and arr.shape[1] in (3, 6, 9)):
                     return arr
 
-            return [normalize_data(d) for d in data]  # type: ignore [arg-type, return-value]
+            return [normalize(d) for d in data]
 
         if isinstance(data, int):
             return float(data)
 
-        return normalize_data(data)
+        return normalize(data)
 
     if isinstance(data, np.ndarray):
-        ret = data.tolist()
-        assert isinstance(ret, (int, float, list))
-        return ret  # type: ignore [return-value]
+        ret2 = data.tolist()
+        assert isinstance(ret2, (int, float, list))
+        return ret2
 
     if (
         not isinstance(data, DimensionSet)
@@ -111,27 +135,31 @@ def normalize_data(
         return DimensionSet(*data)
 
     if keywords is None and isinstance(data, tuple) and len(data) == 2:
-        k, v = data
-        assert not isinstance(k, Mapping)
-        return (  # type: ignore [return-value]
-            normalize_keyword(k),  # type: ignore [arg-type]
-            normalize_data(v) if not isinstance(v, Mapping) else v,  # type: ignore [arg-type]
-        )
+        k2, v2 = data
+        if isinstance(k2, Mapping):
+            msg = "Keyword in keyword entry cannot be a dictionary"
+            raise ValueError(msg)
+        k2 = normalize(k2, bool_ok=False)
+        v2 = normalize(v2)
+        return (k2, v2)
 
     if (
         is_sequence(data)
         and not isinstance(data, DimensionSet)
         and not isinstance(data, tuple)
     ):
-        return [normalize_data(d) for d in data]  # type: ignore [arg-type, return-value]
+        return [normalize(d) for d in data]
 
     if isinstance(data, tuple) and not isinstance(data, DimensionSet):
-        return tuple(normalize_data(d, keywords=keywords) for d in data)  # type: ignore [misc]
+        return tuple(normalize(d, keywords=keywords) for d in data)
 
     if isinstance(data, str):
-        s = loads(data, keywords=keywords)
-        if isinstance(s, (str, tuple, bool)):
-            return s
+        with contextlib.suppress(ValueError, KeyError):
+            s = Parsed(data)[()]
+            if not bool_ok and isinstance(s, bool):
+                return data
+            if isinstance(s, (str, tuple, bool)):
+                return s
 
     if isinstance(
         data,
@@ -143,15 +171,6 @@ def normalize_data(
     raise TypeError(msg)
 
 
-def normalize_keyword(data: DataLike) -> Data:
-    ret = normalize_data(data)
-
-    if isinstance(data, str) and isinstance(ret, bool):
-        return data
-
-    return ret
-
-
 def dumps(
     data: DataLike | StandaloneDataLike | KeywordEntryLike | SubDictLike,
     *,
@@ -159,7 +178,7 @@ def dumps(
     header: SubDictLike | None = None,
     tuple_is_keyword_entry: bool = False,
 ) -> bytes:
-    data = normalize_data(data, keywords=keywords)  # type: ignore [arg-type, misc]
+    data = normalize(data, keywords=keywords)
 
     if isinstance(data, Mapping):
         return (
